<?php

/**
 * Provide a widget list for the plugin
 *
 * This file is used to create a wordpress styled paginated list for the widgets.
 *
 * @link       http://estatebud.com
 * @since      1.0.0
 *
 * @package    Estatebud
 * @subpackage Estatebud/admin/partials
 */

if (!class_exists('WP_List_Table')) {
    require_once (ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class Estatebud_Widgets_List extends WP_List_Table
{
    function __construct()
    {
        global $status, $page;

        //Set parent defaults
        parent::__construct(array(
            'singular' => 'widget',    //singular name of the listed records
            'plural' => 'widgets',     //plural name of the listed records
            'ajax' => true                //does this table support ajax?
        ));
    }

    function column_default($item, $column_name)
    {
        switch ($column_name) {
            case 'label' :
                return $item[$column_name];
            default :
                return "";
        }
    }

    function column_label($item)
    {
        return $item["label"];
    }

    function column_actions($item)
    {
       return "<button type='button' class='button' onclick='return estatebud.show_widget_edit_form(\"".$item["widget_key"]."\",\"".wp_create_nonce("estatebud_get_widget", "widget_security")."\")'>" .
       __('Edit') . "</button>&nbsp;<button type='button' class='button' onclick='return estatebud.delete_widget(\"".$item["widget_key"]."\", \"".wp_create_nonce("estatebud_delete_widget", "widget_security")."\")'>" . __('Delete') . "</button>";
    }

    function column_cb($item)
    {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],
            /*$2%s*/ $item['widget_key']
        );
    }

    function get_columns()
    {
        $columns = array(
            'cb' => '<input type="checkbox" />', //Render a checkbox instead of text
            'label' => __('Search Box Label', 'estatebud'),
            'actions' => __('Actions'),
        );
        return $columns;
    }

    function get_sortable_columns()
    {
        $sortable_columns = array(
            'label' => array('label', false),
        );
        return $sortable_columns;
    }

    function get_bulk_actions()
    {
        $actions = array(
            'delete' => __('Delete')
        );
        return $actions;
    }

    function process_bulk_action()
    {
        global $wpdb;
        $prefix = $wpdb->prefix;
        //Detect when a bulk action is being triggered...
        if ('delete' === $this->current_action()) {
            if (isset($_POST["action2"])) {
                $items = get_option("estatebud_widget_settings", false);
                if($items){
                    foreach($_POST["widget"] as $widget)
                        unset($items[$widget]);
                    update_option( "estatebud_widget_settings", $items );
                }
            }
            elseif (isset($_GET["action"])) {
                $items = get_option("estatebud_widget_settings", false);
                if($items){
                    unset($items[$_GET["id"]]);
                    update_option( "estatebud_widget_settings", $items );
                }
            }
        }
    }

    function prepare_items()
    {
        global $wpdb; //This is used only if making any database queries

        $per_page = 10;

        $columns = $this->get_columns();
        $hidden = [];
        $sortable = $this->get_sortable_columns();

        $this->_column_headers = array($columns, $hidden, $sortable, "cb");

        $this->process_bulk_action();

        $current_page = $this->get_pagenum();

        $data = $this->query_db( ( ($current_page - 1) * $per_page), $per_page);

        $total_items = $this->query_db_count();

        $this->items = $data;

        $this->set_pagination_args(array(
            'total_items' => $total_items,                  //WE have to calculate the total number of items
            'per_page' => $per_page,                     //WE have to determine how many items to show on a page
            'total_pages' => ceil($total_items / $per_page)   //WE have to calculate the total number of pages
        ));
    }

    function query_db($offset, $limit)
    {
        $items = get_option("estatebud_widget_settings", false);
        if($items){
            if(isset($_GET["s"])){
                $query = sanitize_text_field($_GET["s"]);
                $items = array_filter($items, function($item) use ($query){
                    return strpos(strtolower($item["label"]), strtolower($query)) > 0;
                });
            }
            usort($items, function ($a1, $a2){
                if ($a1["create"] == $a2["create"]) return 0;
                return ($a1["create"] < $a2["create"]) ? -1 : 1;
            });
            return array_slice($items, $offset, $limit, true);
        }
        return [];
    }

    function query_db_count()
    {
        $items = get_option("estatebud_widget_settings", false);
        if($items){
            return count($items);
        }
        return false;
    }

    /**
     * Display the pagination.
     * This is a plain copy of parent::pagination() with a few tricks to edit
     * the links that will trigger jQuery. I tend to prefer this way rather than
     * have a heavier jQuery part playing with classes to get the right links.
     */
    function pagination($which)
    {
        if (empty($this->_pagination_args))
            return;

        extract($this->_pagination_args, EXTR_SKIP);

        $output = '<span class="displaying-num">' . sprintf(_n('1 item', '%s items', $total_items), number_format_i18n($total_items)) . '</span>';

        $current = $this->get_pagenum();

        $current_url = set_url_scheme('http'.(is_ssl() ? "s" : "").'://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);

        $current_url = remove_query_arg(array('hotkeys_highlight_last', 'hotkeys_highlight_first'), $current_url);

        $page_links = array();

        $disable_first = $disable_last = '';
        if ($current == 1)
            $disable_first = ' disabled';
        if ($current == $total_pages)
            $disable_last = ' disabled';

        $page_links[] = sprintf(
            "<a class='%s' title='%s' href='%s' data-nav='%s' data-nav-direction='first' data-nav-paged='%s'>%s</a>",
            'first-page' . $disable_first,
            esc_attr__('Go to the first page'),
            esc_url(remove_query_arg('paged', $current_url)), ('' == $disable_first ? 'true' : 'false'),
            1,
            '&laquo;'
        );

        $page_links[] = sprintf(
            "<a class='%s' title='%s' href='%s' data-nav='%s' data-nav-direction='prev' data-nav-paged='%s'>%s</a>",
            'prev-page' . $disable_first,
            esc_attr__('Go to the previous page'),
            esc_url(add_query_arg('paged', max(1, $current - 1), $current_url)), ('' == $disable_first ? 'true' : 'false'),
            max(1, $current - 1),
            '&lsaquo;'
        );

        if ('bottom' == $which)
            $html_current_page = $current;
        else
            $html_current_page = sprintf(
            "<input class='current-page' title='%s' type='text' name='paged' value='%s' size='%d' />",
            esc_attr__('Current page'),
            $current,
            strlen($total_pages)
        );

        $html_total_pages = sprintf("<span class='total-pages'>%s</span>", number_format_i18n($total_pages));
        $page_links[] = '<span class="paging-input">' . sprintf(_x('%1$s of %2$s', 'paging'), $html_current_page, $html_total_pages) . '</span>';

        $page_links[] = sprintf(
            "<a class='%s' title='%s' href='%s' data-nav='%s' data-nav-direction='next' data-nav-paged='%s'>%s</a>",
            'next-page' . $disable_last,
            esc_attr__('Go to the next page'),
            esc_url(add_query_arg('paged', min($total_pages, $current + 1), $current_url)), ('' == $disable_last ? 'true' : 'false'),
            min($total_pages, $current + 1),
            '&rsaquo;'
        );

        $page_links[] = sprintf(
            "<a class='%s' title='%s' href='%s' data-nav='%s' data-nav-direction='last' data-nav-paged='%s'>%s</a>",
            'last-page' . $disable_last,
            esc_attr__('Go to the last page'),
            esc_url(add_query_arg('paged', $total_pages, $current_url)), ('' == $disable_last ? 'true' : 'false'),
            $total_pages,
            '&raquo;'
        );

        $pagination_links_class = 'pagination-links';
        if (!empty($infinite_scroll))
            $pagination_links_class = ' hide-if-js';
        $output .= "\n<span class='$pagination_links_class'>" . join("\n", $page_links) . '</span>';

        if ($total_pages)
            $page_class = $total_pages < 2 ? ' one-page' : '';
        else
            $page_class = ' no-pages';

        $this->_pagination = "<div class='tablenav-pages{$page_class}'>$output</div>";

        echo $this->_pagination;
    }

    /**
     * Handle an incoming ajax request (called from admin-ajax.php)
     */
    function ajax_response()
    {
        $this->prepare_items();

        extract($this->_args);
        extract($this->_pagination_args, EXTR_SKIP);

        ob_start();
        if (!empty($_REQUEST['no_placeholder']))
            $this->display_rows();
        else
            $this->display_rows_or_placeholder();

        $rows = ob_get_clean();

        ob_start();
        $this->pagination('top');
        $pagination_top = ob_get_clean();

        ob_start();
        $this->pagination('bottom');
        $pagination_bottom = ob_get_clean();

        $response = array('rows' => $rows);
        $response['pagination']['top'] = $pagination_top;
        $response['pagination']['bottom'] = $pagination_bottom;

        if (isset($total_items))
            $response['total_items_i18n'] = sprintf(_n('1 item', '%s items', $total_items), number_format_i18n($total_items));

        if (isset($total_pages)) {
            $response['total_pages'] = $total_pages;
            $response['total_pages_i18n'] = number_format_i18n($total_pages);
        }

        die(json_encode($response));
    }
}

/***************************** AJAX CALLBACK ********************************
 ****************************************************************************
 * This function loads the Custom List Table Class and calls ajax_response method
 */
function estatebud_widgets_get_paginated_content()
{
    $wp_list_table = new Estatebud_Widgets_List();
    $wp_list_table->ajax_response();
}
add_action('wp_ajax_estatebud_widgets_get_paginated_content', 'estatebud_widgets_get_paginated_content');