<?php
class EstatebudAPI
{

    private $api_key = null;
    private $version = "v1";
    private $url = "https://api.estatebud.com";
    private $locale = null;
    public $auth_info = null;


    function __construct($api_key, $locale = "en_US")
    {
        $this->api_key = trim($api_key);

        $this->cache = new Estatebud_Cache();

        if ($locale == 'el') {
            $locale = "el_GR";
        }

        $this->auth_info = get_option("estatebud_api_whois", null);

        if($this->auth_info == null){
            $this->check_authentication();
        }

        $this->locale = str_replace("_", "-", $locale);
    }

    /**
     * Checks whether the API key is valid/invalid
     *
     * @param string $api_key
     * @return bool Api key status
     */
    function check_authentication()
    {
        $result = $this->get_json("whois");
        if ($result) {
            if (isset($result->error)) {
                update_option("estatebud_api_whois", null);
                $this->auth_info = null;
                return false;
            } else {
                $this->auth_info = $result;
                update_option("estatebud_api_whois", $result);
                return true;
            }
        } else {
            echo __("Estatebud API Server: Communication Error", "estatebud");
            return false;
        }
    }

    /**
     * Endpoints
     */

    /**
     * Get Property Categories
     *
     * @return object property categories
     */

    public function get_property_categories()
    {
        return $this->cache->remember("get_property_categories", $this->locale, 72 * 60 * 60, function($locale) {
            return $this->get_json("getPropertyCategories", ["language" => $locale]);
        }, ["locale" => $this->locale]);
    }

    /**
     * Get Property Types
     *
     * @return object property types
     */
    public function get_property_types()
    {
        return $this->cache->remember("get_property_types", $this->locale, 3 * 60 * 60, function($locale) {
            return $this->get_json("getPropertyTypes", ["language" => $locale]);
        }, ["locale" => $this->locale]);
    }

    public function get_all_currencies() {
        return $this->cache->remember("get_currencies", $this->locale, 24 * 60 * 60, function(){
            return $this->get_json("getCurrency");
        });
    }

    public function get_currency_rates() {
        $selected_currencies = get_option("estatebud_selected_currencies", []);
        $base_currency = strtoupper($this->auth_info->currency);
        $all_currencies = array_merge($selected_currencies, [$base_currency]);
        $currency_hash = md5(implode(",", $all_currencies));
        return $this->cache->remember("get_currencies_" . $currency_hash, $this->locale, 8 * 60 * 60, function($base_currency, $selected_currencies){
            // delete other cached currencies
            global $wpdb;
            $wpdb->query("DELETE from {$wpdb->prefix}options where option_name like 'cache_estatebud_get_currencies_%'");
            return $this->get_json("getCurrency", ["base" => $base_currency, "symbols" => implode(",", $selected_currencies), "amount" => 1]);
        },[$base_currency, $all_currencies]);
    }

    /**
     * Get Property Conditions
     *
     * @return object property conditions
     */
    public function get_property_statuses()
    {
        return $this->cache->remember("get_property_statuses", $this->locale, 48 * 60 * 60, function($locale) {
            return $this->get_json("getPropertyStatuses", ["language" => $locale]);
        }, ["locale" => $this->locale]);
    }

    /**
     * Get Property Areas
     *
     * @param string $country
     * @return object property areas of country
     */
    public function get_property_areas($country = null)
    {
        return $this->cache->remember("get_property_areas_$country", $this->locale, 2 * 60 * 60, function($country, $locale) {
            return $this->get_json("getAreas", ["country" => $country, "language" => $locale], true);
        }, ["country" => $country, "locale" => $this->locale]);
    }

    /**
     * Get Property Details availability + price within the date range
     *
     * @param int $id
     * @param date $date_from
     * @param date $date_to
     * @return object single property info
     */
    public function get_property($id, $date_from, $date_to)
    {
        return $this->get_json("getProperty/$id", ["date_from" => $date_from, "date_to" => $date_to, "language" => $this->locale]);
    }

    /**
     * Get PDF Brochure template options
     *
     * @return object PDF Brochure templates
     */
    public function get_property_pdf_templates()
    {
        return $this->get_json("getPropertyPDFTemplates");
    }

    /**
     * Returns the property brochure pdf
     *
     * @param int $property_id
     * @param boolean $agent_info
     * @param boolean $branding_free
     * @return object PDF brochure
     */
    public function get_property_pdf($property_id, $template_id, $agent_info, $branding_free)
    {
        return $this->get_json("getPropertyPDF", array_filter([
            "id" => $property_id,
            "template_id" => $template_id,
            "agent_details" => $agent_info,
            "branding_free" => $branding_free,
            "language" => $this->locale
        ]));
    }

    /**
     * Get Custom Property Labels
     *
     * @return object property labels array
     */
    public function get_property_labels()
    {
        return $this->get_json("getPropertyLabels");
    }

    /**
     * Get CRM Pipelines
     *
     * @return object pipeline array
     */
    public function get_pipelines()
    {
        return $this->get_json("getPipelines");
    }

    /**
     * Creates a new deal record on CRM
     *
     * @param int $pipeline_id
     * @param int $stage_id
     * @param string $name
     * @param string $email
     * @param string $phone
     * @param string $note
     * @return void
     */
    public function new_deal($pipeline_id, $stage_id, $name, $email, $phone, $property_id, $note, $currency, $value)
    {
        return $this->post("newDeal", [
            "pipeline_id" => $pipeline_id,
            "stage_id" => $stage_id,
            "name" => $name,
            "email" => $email,
            "phone" => $phone,
            "property_id" => $property_id,
            "note" => $note,
            "currency" => $currency,
            "value" => $value
        ]);
    }

    /**
     * Property Listing with filters
     *
     * @param int $property_id
     * @param int $area
     * @param string $category
     * @param string $type
     * @param string $label
     * @param int $city
     * @param date $date_from
     * @param date $date_to
     * @param bool $public
     * @param bool $featured
     * @param string $title_deed = yes, yes_share_of_land, final_approval, no
     * @param integer $bedrooms_min
     * @param integer $bedrooms_max
     * @param integer $bathrooms_min
     * @param integer $bathrooms_max
     * @param integer $covered_sqm_min
     * @param integer $covered_sqm_max
     * @param integer $price_min
     * @param integer $price_max
     * @param string $status = resale, key_ready, under_construction, off_plan
     * @param bool $sea_view
     * @param bool $sea_front
     * @param bool $swimming_pool
     * @param bool $walking_distance_to_beach
     * @param integer $rent_min
     * @param integer $rent_max
     * @param string $furnished = $furnished = furnished, partiallyfurnished, unfurnished
     * @param string $sort = availability, property_id (default), price, rent
     * @param string $sort_type = DESC, ASC
     * @param integer $offset
     * @param integer $limit
     * @param boolean $sold
     * @param boolean $rented
     * @param boolean $map_mode
     * @param float $lat
     * @param float $lng
     * @param float $radius
     *
     * @return void
     */
    public function get_properties($id = null, $property_id = null, $area = null, $category = null, $type = null, $label = null, $city = null, $date_from = null, $date_to = null, $public = null, $featured = null, $title_deed = null, $bedrooms_min = null, $bedrooms_max = null, $bathrooms_min = null, $bathrooms_max = null, $covered_m2_min = null, $covered_m2_max = null, $covered_ft2_min = null, $covered_ft2_max = null, $price_min = null, $price_max = null, $status = null, $sea_view = null, $sea_front = null, $swimming_pool = null, $walking_distance_to_beach = null, $rent_min = null, $rent_max = null, $furnished = null, $sort = "property_id", $sort_type = "DESC", $offset = 0, $limit = 12, $sold = true, $rented = true, $map_mode = false, $lat = 0, $lng = 0, $radius = 0)
    {
        $ref = new ReflectionMethod(EstatebudAPI::class, "get_properties");
        $parameters = $ref->getParameters();
        $query = [];
        foreach ($parameters as $parameter) {
            if ($parameter->name !== "offset" && $parameter->name !== "limit") {
                $value = ${$parameter->name};
                if ($value !== null && $value !== $parameter->getDefaultValue()) {
                    $query[$parameter->name] = ${$parameter->name};
                }
            }
        }
        $query["title_deed"] = !empty($query["title_deed"]) ? "yes" : false;
        $query["limit"] = $offset . "," . $limit;
        $query["public"] = 1;
        $query["sort"] = $sort;
        $query["sort_type"] = $sort_type;
        $query["language"] = $this->locale;

        return $this->get_json("getProperties", $query);
    }

    /**
     * API Fetching method
     *
     * @param string    $endpoint    The API endpoint
     * @param array     $query       The query parameters
     * @return object                The query result
     */
    function get_json($endpoint, $query = [], $as_array = false)
    {

        $query = array_filter($query, function ($d) {
            $d = is_array($d) ? trim(implode(",", $d)) : $d;
            return is_string($d) ? strlen($d) : !empty($d);
        });

        $args = array(
            'headers' => array(
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Basic ' . $this->api_key,
            ),
            'method' => 'GET',
            'timeout' => 45,
            'sslverify' => false,
        );

        $url = "$this->url/$this->version/$endpoint" . (count($query) > 0 ? "?" . urldecode(http_build_query($query)) : "");

        // var_dump($url);

        // $time = microtime(true);

        $response = wp_remote_request($url, $args);
        $response_code = wp_remote_retrieve_response_code($response);

        if($response_code == 200){
            $body = wp_remote_retrieve_body($response);
            // var_dump(json_decode($body));
            // error_log("$endpoint API Call Took " . (microtime(true) - $time) . "secs");
            return json_decode($body, $as_array);
        } else {
            return false;
        }
    }

    function post($endpoint, $query = [], $as_array = false)
    {

        $query = array_filter($query, function ($d) {
            $d = is_array($d) ? trim(implode(",", $d)) : $d;
            return is_string($d) ? strlen($d) : !empty($d);
        });

        $args = array(
            'headers' => array(
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Basic ' . $this->api_key,
            ),
            'method' => 'POST',
            'timeout' => 45,
            'sslverify' => false,
            'body' => json_encode($query),
        );

        $url = "$this->url/$this->version/$endpoint" . (count($query) > 0 ? "?" . urldecode(http_build_query($query)) : "");

        // var_dump($url);

        // $time = microtime(true);

        $response = wp_remote_request($url, $args);
        $response_code = wp_remote_retrieve_response_code($response);

        if($response_code == 200){
            $body = wp_remote_retrieve_body($response);
            // var_dump(json_decode($body));
            // error_log("$endpoint API Call Took " . (microtime(true) - $time) . "secs");
            return json_decode($body, $as_array);
        } else {
            return false;
        }
    }
}

?>
