<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://estatebud.com
 * @since      1.0.0
 *
 * @package    Estatebud
 * @subpackage Estatebud/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Estatebud
 * @subpackage Estatebud/includes
 * @author     Estatebud
 */
class Estatebud
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Estatebud_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $id    The string used to uniquely identify this plugin.
	 */
	protected $id;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The translator instance
	 *
	 * @since	4.2.4
	 * @access 	protected
	 * @var 	Estatebud_i18n	$i18n_instance	The translation class instance
	 */
	protected $i18n_instance;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if (defined('ESTATEBUD_PLUGIN_VERSION')) {
			$this->version = ESTATEBUD_PLUGIN_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->id = 'estatebud';

		$this->load_dependencies();
		$this->set_locale();
		if (is_admin()) $this->define_admin_hooks();
		else $this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Estatebud_Loader. Orchestrates the hooks of the plugin.
	 * - Estatebud_i18n. Defines internationalization functionality.
	 * - Estatebud_Admin. Defines all hooks for the admin area.
	 * - Estatebud_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-i18n.php';

		/**
		 * The class responsible for displaying and customizing the widget
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-widget.php';

		/**
		 * The class responsible for displaying and customizing the slider widget
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-slider-widget.php';

		/**
		 * The class responsible for displaying and customizing the searchbox widget
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-search-widget.php';

		/**
		 * The class responsible for displaying and customizing the currency list widget
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-currency-widget.php';

		/**
		 * The class responsible for the cache operations
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-cache.php';

		/**
		 * The class responsible for defining all API interfacing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-estatebud-api.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-estatebud-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-estatebud-public.php';

		$this->loader = new Estatebud_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Estatebud_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$this->i18n_instance = new Estatebud_i18n();

		$this->loader->add_action('plugins_loaded', $this->i18n_instance, 'load_plugin_textdomain');

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{

		$plugin_admin = new Estatebud_Admin($this->get_id(), $this->get_version());

		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Estatebud_Public($this->get_id(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_id()
	{
		return $this->id;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Estatebud_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}

	/**
	 * Retrieve the translation instance of the plugin
	 *
	 * @since 		4.2.4
	 * @access 		public
	 * @return 		Estatebud_i18n	The class instance of created Estatebud_i18n class
	 */
	public function get_translator()
	{
		return $this->i18n_instance;
	}

}
