<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://estatebud.com
 * @since      1.0.0
 *
 * @package    Estatebud
 * @subpackage Estatebud/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Estatebud
 * @subpackage Estatebud/public
 * @author     Estatebud
 */
class Estatebud_Public
{
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $id    The ID of this plugin.
     */
    private $id;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * The page type flag of details.
     *
     * @since    1.0.0
     * @access   private
     * @var      boolean    $is_details_page   Shows the page type is property details page.
     */
    private $is_details_page;

    /**
     * The page type flag of listings.
     *
     * @since    1.0.0
     * @access   private
     * @var      boolean    $is_listings_page   Shows the page type is property listings page.
     */
    private $is_listing_page;

    /**
     * The page type flag of maps
     *
     * @since     3.0.0
     * @access    private
     * @var       boolean    $is_maps_page      Shows the page tpe is mapped property listings page.
     */
    private $is_maps_page;

    /**
     * Global member to hold fetched details
     *
     * @since     3.0.0
     * @access    private
     * @var       object     $details           Holds the property data for use in between methods
     */
    private $details;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $id         The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($id, $version)
    {
        $this->id = $id;
        $this->version = $version;

        $this->is_details_page = false;
        $this->is_listing_page = false;
        $this->is_maps_page = false;
        $this->details = false;
		
        $this->register_shortcodes();
        $this->register_detail_shortcode();
        $this->register_favorites_shortcode();
		
		add_filter('document_title_parts', array($this, 'estatebud_filter_wp_title'));
		add_filter('template_redirect', array($this, 'estatebud_get_page_type'));
		
		add_filter('wpseo_canonical', function($def) { $res = $this->add_estatebud_meta('canonical'); if($res) return $res; else return $def; });
		add_filter('wpseo_title', function($def) { $res = $this->add_estatebud_meta('title'); if($res) return $res; else return $def; });
		add_filter('wpseo_metadesc', function($def) { $res = $this->add_estatebud_meta('metadesc'); if($res) return $res; else return $def; });
		
		add_filter('wpseo_opengraph_site_name', function($def) { $res = $this->add_estatebud_meta('og:site_name'); if($res) return $res; else return $def; });
		add_filter('wpseo_opengraph_title', function($def) { $res = $this->add_estatebud_meta('og:title'); if($res) return $res; else return $def; });
		add_filter('wpseo_opengraph_desc', function($def) { $res = $this->add_estatebud_meta('og:desc'); if($res) return $res; else return $def; });
		add_filter('wpseo_opengraph_url', function($def) { $res = $this->add_estatebud_meta('og:url'); if($res) return $res; else return $def; });
		add_filter('wpseo_add_opengraph_images', function($def) { $res = $this->add_estatebud_meta('og:image'); if($res) return $res; else return $def; });
		
		add_filter('wpseo_locale', function($def) { $res = $this->add_estatebud_meta('locale'); if($res) return $res; else return $def; });
		
		add_filter('wp_head', array($this, 'add_estatebud_meta'));
		add_filter('the_title', array($this, 'estatebud_remove_title_from_properties'));
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        wp_enqueue_style('font-awesome', '//use.fontawesome.com/releases/v5.6.3/css/all.css');
        wp_enqueue_style($this->id, plugin_dir_url(__FILE__) . 'css/estatebud-public.css', array(), $this->version, 'all');
        wp_enqueue_style($this->id . "-theme-color", plugin_dir_url(__FILE__) . 'css/estatebud-dynamic.php', array(), null, 'all');
        if ($this->is_listing_page) {
            wp_enqueue_style($this->id . "-select", plugin_dir_url(__FILE__) . 'css/sumoselect.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-override", plugin_dir_url(__FILE__) . 'css/estatebud-sumoselect.css', array(), $this->version, 'all');
        }
        if ($this->is_details_page) {
            wp_enqueue_style($this->id . "-slider", plugin_dir_url(__FILE__) . 'css/slick.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-slider-override", plugin_dir_url(__FILE__) . 'css/estatebud-slick.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-share", plugin_dir_url(__FILE__) . 'css/jssocials.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-fancybox", plugin_dir_url(__FILE__) . 'css/jquery.fancybox.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-share-theme", plugin_dir_url(__FILE__) . 'css/jssocials-theme-minima.css', array(), $this->version, 'all');
        }
        if ($this->is_maps_page) {
            wp_enqueue_style($this->id . "-select", plugin_dir_url(__FILE__) . 'css/sumoselect.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-override", plugin_dir_url(__FILE__) . 'css/estatebud-sumoselect.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-maps", plugin_dir_url(__FILE__) . 'css/estatebud-public-maps.css', array(), $this->version, 'all');
            wp_enqueue_style($this->id . "-slider", plugin_dir_url(__FILE__) . 'css/slick.css', array(), $this->version, 'all');
        }
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
        wp_enqueue_script($this->id, plugin_dir_url(__FILE__) . 'js/estatebud-public.js', array('jquery'), $this->version, false);
        if ($this->is_listing_page) {
            wp_enqueue_script($this->id . "-select", plugin_dir_url(__FILE__) . 'js/sumoselect.full.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-deserialize", plugin_dir_url(__FILE__) . 'js/jquery.deserialize.js', array('jquery'), $this->version, false);
        }
        if ($this->is_details_page) {
            wp_enqueue_script($this->id . "-slider", plugin_dir_url(__FILE__) . 'js/slick.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-share", plugin_dir_url(__FILE__) . 'js/jssocials.min.js', array('jquery'), $this->version, false);

            if (get_option("estatebud_disable_fancybox", false) == false) {
                if (!wp_script_is('jquery.fancybox.min.js')) {
                    wp_enqueue_script($this->id . "-fancybox", plugin_dir_url(__FILE__) . 'js/jquery.fancybox.min.js', array('jquery'), $this->version, false);
                }
            }
            wp_enqueue_script($this->id . "-maps", "//maps.googleapis.com/maps/api/js?language=" . get_locale() . "&callback=estatebud_createMap&key=" . trim(get_option("estatebud_gmaps_api_key", "")), "", "", true);

            if(get_option("estatebud_use_recaptcha")){
                if(get_option("recaptcha3_sitekey_value")){
                    wp_enqueue_script($this->id . "-recaptcha", "//www.google.com/recaptcha/api.js?render=" . get_option("recaptcha3_sitekey_value"), "", "", true);
                }else{
                    wp_enqueue_script($this->id . "-recaptcha", "//www.google.com/recaptcha/api.js", "", "", true);
                }
            }

            if (get_option("estatebud_show_qr_code", false) == true) {
                wp_enqueue_script($this->id . "-qrcode", plugin_dir_url(__FILE__) . 'js/qrcode.min.js', array('jquery'), $this->version, false);
            }
        }
        if ($this->is_maps_page) {
            wp_enqueue_script($this->id . "-select", plugin_dir_url(__FILE__) . 'js/sumoselect.full.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-deserialize", plugin_dir_url(__FILE__) . 'js/jquery.deserialize.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-slider", plugin_dir_url(__FILE__) . 'js/slick.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->id . "-maps-clusterer", plugin_dir_url(__FILE__) . 'js/markerclusterer.js', array('jquery', $this->id), $this->version, false);
            wp_enqueue_script($this->id . "-maps-lib", plugin_dir_url(__FILE__) . 'js/estatebud-public-maps.js', array('jquery', $this->id), $this->version, false);
            wp_enqueue_script($this->id . "-maps", "//maps.googleapis.com/maps/api/js?language=" . get_locale() . "&callback=estatebud_createMap&libraries=places,geometry&key=" . trim(get_option("estatebud_gmaps_api_key", "")), "", "", false);
            wp_enqueue_script($this->id . "-maps-marker", plugin_dir_url(__FILE__) . 'js/markerwithlabel.min.js', array('jquery', $this->id), $this->version, false);
            wp_localize_script($this->id . "-maps-lib", 'estatebud_maps', array(
                'nonce' => wp_create_nonce("estatebud_get_infowindow")
            ));
        }
        wp_localize_script($this->id, 'estatebudL10n', array(
            'show_less' => estatebud__("app.simple.search"),
            'show_more' => estatebud__("app.advanced.search"),
            'any' => estatebud__("app.any"),
            'AddToFavorites' => estatebud__("app.add.to.favorites"),
            'RemoveFromFavorites' => estatebud__("app.remove.from.favorites")
        ));

        wp_localize_script($this->id, 'estatebud', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'version' => $this->version,
            'is_details' => $this->is_details_page ? "true" : "false",
            'is_listing' => $this->is_listing_page ? "true" : "false",
            'is_maps' => $this->is_maps_page ? "true" : "false",
            'plugin_url' => plugin_dir_url(__FILE__),
            'lang' => get_locale()
        ));
    }

    /**
     * If the page contains the detail shortcode, it rewrites the title with property number
     *
     * @param [type] $title_parts
     * @return void
     */
    public function estatebud_filter_wp_title($title_parts)
    {
        global $wp_query;
        if ($this->is_details_page) {
            $title_parts["title"] = sprintf(__("Property #%d", "estatebud"), isset($_GET["id"]) ? intval($_GET["id"]) : $wp_query->query["page"]);
            unset($title_parts["page"]);
            $wp_query->query["remove_title"] = true;
        } else if ($this->is_listing_page || $this->is_maps_page) {
            $wp_query->query["remove_title"] = true;
        }
        return $title_parts;
    }

    /**
     * Gets the page type (listing or details) prior to script/style enqueues.
     *
     * @return void
     */
    public function estatebud_get_page_type()
    {
        if (is_page()) {
            global $shortcode_tags, $wp_query;
            $estatebud_shortcodes = array_filter(array_keys($shortcode_tags), function ($shortcode) {
                return substr($shortcode, 0, 9) == "estatebud";
            });
            foreach ($estatebud_shortcodes as $shortcode) {
                if (has_shortcode($wp_query->post->post_content, $shortcode) !== false) {
                    if ($shortcode === "estatebud_detail") {
                        $this->is_details_page = true;
                        if(isset($_GET["pdf"]) && $_GET["pdf"] == "true")
                        {
                            $this->estatebud_download_pdf();
                        }
                        remove_action('wp_head', '_wp_render_title_tag', 1);
                    } else {
                        if(preg_match_all("/\[(".$shortcode."[^\]]+)]/",$wp_query->post->post_content, $matches)){
                            foreach($matches[1] as $match){
                                $attrs = shortcode_parse_atts( $match );
                                if(isset($attrs["is_widget"]) == false || $attrs["is_widget"] == false){
                                    $this->is_listing_page = true;
                                    if(get_option("estatebud_show_listing_map", false))
                                    {
                                        $this->is_maps_page = true;
                                    }
                                    return;
                                }
                            }
                        }else{
                            $this->is_listing_page = true;
                            if(get_option("estatebud_show_listing_map", false))
                            {
                                $this->is_maps_page = true;
                            }
                            return;
                        }
                    }
                    return;
                } else if (has_shortcode($wp_query->post->post_content, $shortcode . "_map") !== false) {
                    $this->is_maps_page = true;
                    return;
                }
            }
        }
    }

    public function estatebud_download_pdf()
    {
        global $wp_query;
        if (isset($wp_query->query["is_property_details"])) {
            $property_id = $wp_query->query["property_id"];
            $pdf_template_id = get_option("estatebud_pdf_template_id", 1);
            $enable_agent_info = get_option("estatebud_pdf_show_agent", false);
            $branding_free = get_option("estatebud_pdf_branding_free", false);
            $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), isset($_GET["lang"]) ? $_GET["lang"] : get_locale());

			if(!$this->details) {
				$this->details = $estatebud_api->get_property($property_id, null, null);
			}

			$details = $this->details;

            $pdf = $estatebud_api->get_property_pdf($details->id, $pdf_template_id, $enable_agent_info, $branding_free);
            header("Content-type: application/pdf");
            header("Content-Disposition: attachment; filename={$details->reference_id}.pdf");
            echo base64_decode($pdf->file);
            die();
        }
    }

    /**
     * Removes title from property pages
     */
    public function estatebud_remove_title_from_properties($title)
    {
        global $wp_query;
        if (isset($wp_query->query["remove_title"]) || $this->is_details_page) {
            if (!array_intersect(["widget", "wp_nav_menu"], array_column(debug_backtrace(), "function")))
                return "";
        }
        return $title;
    }

    /**
     * Registers the dynamic shortcodes
     *
     * @return void
     */
    public function register_shortcodes()
    {
        $shortcodes = get_option("estatebud_shortcode_settings", []);
        foreach ($shortcodes as $shortcode_key => $data) {

            /**
             * Listing Shortcodes
             */
            add_shortcode("estatebud_" . $shortcode_key, function ($atts) use ($data) {
                $is_widget = isset($atts["is_widget"]);
                if ($is_widget) {
                    $property_limit = isset($atts["limit"]) ? $atts["limit"] : 5;
                    $layout = isset($atts["layout"]) ? $atts["layout"] : "horizontal";
                    $visible_count = isset($atts["visiblecount"]) ? $atts["visiblecount"] : 3;
                    $scroll_count = isset($atts["scrollcount"]) ? $atts["scrollcount"] : 3;
                    $autoplay = isset($atts["autoplay"]) ? $atts["autoplay"] : false;
                    $infinite = isset($atts["infinite"]) ? $atts["infinite"] : true;
                    $autoplay_duration = isset($atts["autoplayduration"]) ? $atts["autoplayduration"] : 2000;
                    $slider_template = isset($atts["slider_template"]) ? $atts["slider_template"] : "slider-multi";
                    $is_slider = isset($atts["is_slider"]) ? $atts["is_slider"] : "";
                }
                $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), get_locale());
                $title_deed = isset($data["title_deed"]) ? trim($data["title_deed"]) : "";
                if (!empty($title_deed)) {
                    $data["title_deed"] = "yes";
                } else {
                    $data["title_deed"] = null;
                }

                if (isset($data["sort"]) && strpos($data["sort"], "|") > 0) {
                    $sorters = explode("|", $data["sort"]);
                    $data["sort"] = $sorters[0];
                    $data["sort_type"] = strtoupper($sorters[1]);
                }

                // apply GET data from searchbox widget
                if (!$is_widget && isset($_GET["from"]) && $_GET["from"] == "searchbox") {
                    $request = $_GET;
                    if (!isset($request["category"])) $request["category"] = [];
                    if (!isset($request["status"])) $request["status"] = [];
                    if (!isset($request["area"])) $request["area"] = [];
                    if (!isset($request["type"])) $request["type"] = [];
                    $data = array_merge($data, $request);
                }

                if($is_widget){
                    $estatebud_list = $this->call_user_func_assoc_array(array($estatebud_api, "get_properties"), $data);
                }else{
                    $estatebud_list = [];
                }
                $request = $data;

                require dirname(__FILE__) . "/partials/estatebud-public-listing.php";

                if ($is_widget) {
                    unset($is_widget, $is_slider, $slider_template, $infinite, $visible_count, $scroll_count, $autoplay_duration, $autoplay, $layout, $property_limit);
                }
            });

            /**
             * Map Shortcodes
             */
            add_shortcode("estatebud_" . $shortcode_key . "_map", function ($atts) use ($data) {

                $position = isset($atts["position"]) && in_array($atts["position"], ["left", "right", "none"]) ? $atts["position"] : "left";
                $sidebar = isset($atts["type"]) && in_array($atts["type"], ["grid", "list"]) ? $atts["type"] : "list";
                $group = isset($atts["group"]) ? filter_var($atts["group"], FILTER_VALIDATE_BOOLEAN) : true;

                $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), get_locale());

                require dirname(__FILE__) . "/partials/estatebud-public-map.php";
            });
        }
    }

    private function call_user_func_assoc_array($callback, $args)
    {
        $ref = new ReflectionMethod(get_class($callback[0]), $callback[1]);
        $parameters = $ref->getParameters();
        $params = [];
        foreach ($parameters as $parameter) {
            $param_name = $parameter->getName();
            $params[$param_name] = isset($args[$param_name]) ? $args[$param_name] : $parameter->getDefaultValue();
            if (is_array($params[$param_name])) $params[$param_name] = implode(",", $params[$param_name]);
        }
        return call_user_func_array($callback, $params);
    }

    /**
     * Registers the detail page shortcode
     *
     * @return void
     */
    public function register_detail_shortcode()
    {
        add_shortcode("estatebud_detail", function () {
            global $wp_query;
            if (isset($wp_query->query["is_property_details"])) {
                $property_id = $wp_query->query["property_id"];
                if(get_option("estatebud_pretty_permalinks", false) == true && strpos($property_id, "-") > 0){
                    $parts = explode("-", $property_id, 1);
                    $last = end($parts);
                    $property_id = $last;
                }
                if ($property_id) {
                    $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), isset($_GET["lang"]) ? $_GET["lang"] : get_locale());

					if(!$this->details) {
						$this->details = $estatebud_api->get_property($property_id, null, null);
					}

					$details = $this->details;

                    if (property_exists($details, "error")) {
                        echo __($details->error, $this->id);
                    } else {
                        $type = get_option("estatebud_details_type", "1");

						$custom_template = dirname(__FILE__) . "/../../estatebud-public-details-custom.php";

						if(file_exists($custom_template)) {
							require_once $custom_template;
						} else {
							$file = dirname(__FILE__) . "/partials/estatebud-public-details-$type.php";
							if (file_exists($file)) {
								require_once $file;
							} else {
								require_once dirname(__FILE__) . "/partials/estatebud-public-details-1.php";
							}
						}
                    }
                }
            }
        });
    }

    /**
     * Registers the favorites page shortcode
     *
     * @return void
     */
    public function register_favorites_shortcode()
    {
        add_shortcode("estatebud_favorites", function () {
            $estatebud_api = new EstatebudAPI(get_option("estatebud_api_key", null), get_locale());
            $data = [];
            $is_widget = false;
            $is_favorites = true;
            $data["property_id"] = implode(",", isset($_COOKIE["estatebud_favorites"]) ? json_decode(stripslashes($_COOKIE["estatebud_favorites"])) : []);
            if(empty($data["property_id"])){
				require dirname(__FILE__) . "/partials/estatebud-public-favorites.php";
            }else{
                $data["is_favorites"] = true;
                $data["sort"] = "property_id";
                $data["sort_type"] = "asc";
                $data["limit"] = 12;
                $_REQUEST = $data;
				$request = $data;
                require dirname(__FILE__) . "/partials/estatebud-public-listing.php";
            }
        });
    }

    /**
     * Adds meta fields for SEO purposes
     *
     * @return void
     */
    public function add_estatebud_meta($source = false)
    {
        global $wp_query;
        if ($this->is_details_page) {
            if (isset($wp_query->query["is_property_details"])) {
                $property_id = $wp_query->query["property_id"];

                if ($property_id) {
                    $api_key = get_option("estatebud_api_key", null);
                    $estatebud_api = new EstatebudAPI($api_key, isset($_GET["lang"]) ? $_GET["lang"] : get_locale());

					if(!$this->details) {
						$this->details = $estatebud_api->get_property($property_id, null, null);
					}

					$details = $this->details;

                    if (property_exists($details, "error")) {
                        echo __($details->error, $this->id);
                    } else {
                        require_once dirname(__FILE__) . "/partials/estatebud-public-details-functions.php";
                        $sitename = get_bloginfo("name");
                        $property_id_string = sprintf(__("Property %s", "estatebud"), $details->reference_id);
                        $area2_name = property_exists($details, "area2_name") ? $details->area2_name : "";
                        $area1_name = property_exists($details, "area1_name") ? $details->area1_name : "";
                        $lang = get_locale();
                        $description = estatebud_get_translation_details(property_exists($details, "description") ? $details->description : false);
                        $link = estatebud_get_property_link($details);
						
						$image="";
						if(property_exists($details, "images") && count($details->images) > 0) {
							foreach($details->images AS $array_image) {
								if($array_image->status != '1') { continue; }
								$image="https:".$array_image->large;
								break;
							}
						}
						
						$title="";
						$og_title="";
						if(property_exists($details, "title")) {
							$title=estatebud_get_translation_details($details->title)." - {$sitename}";
							$og_title=estatebud_get_translation_details($details->title);
						} else {
							$title=$property_id_string.($area2_name ? ", " . $area2_name : "").($area1_name ? ", " . $area1_name : "")." - {$sitename}";
							$og_title=$property_id_string.($area2_name ? ", " . $area2_name : "").($area1_name ? ", " . $area1_name : "");
						}
						
						if($source) {
							switch($source) {
								case "canonical":
									$data = $link;
									break;
								case "title":
									$data = $title;
									break;
								case "metadesc":
									$data = $description;
									break;
								case "og:site_name":
									$data = $sitename;
									break;
								case "og:title":
									$data = $og_title;
									break;
								case "og:desc":
									$data = $description;
									break;
								case "og:url":
									$data = $link;
									break;
								case "og:image":
									echo "<meta property=\"og:image\" content=\"{$image}\" />";
									$data = $image;
									break;
								case "locale":
									$data = $lang;
									break;
							}
							
							return $data;
						} else {
							echo "<title>{$title}</title>";
							echo "<meta name='description' content='{$description}' />";
							echo "<meta property=\"og:url\" content=\"{$link}\" />";
							echo "<meta property=\"og:title\" content=\"{$og_title}\" />";
							echo "<meta property=\"og:description\" content=\"{$description}\" />";
							echo "<meta property=\"og:image\" content=\"{$image}\" />";
							echo "<meta property=\"og:site_name\" content=\"{$sitename}\" />";
						}
                    }
                }
            }
        }
    }
}

function deregister_styles($wp_styles, $keytosearch)
{
    $deregister_styles = false;
    $keylist = [];
    foreach ($wp_styles->registered as $key => $registered) {
        if (strpos($registered->src, $keytosearch) > 0) {
            if (substr($key, 0, 9) != "estatebud") {
                $keylist[] = $key;
            } else {
                $deregister_styles = true;
            }
        }
    }
    if ($deregister_styles) {
        foreach ($keylist as $key) {
            wp_dequeue_style($key);
        }
    }
}

function deregister_scripts($wp_scripts, $keytosearch)
{
    $deregister_scripts = false;
    $keylist = [];
    foreach ($wp_scripts->registered as $key => $registered) {
        if (strpos($registered->src, $keytosearch) > 0) {
            if (substr($key, 0, 9) != "estatebud") {
                $keylist[] = $key;
            } else {
                $deregister_scripts = true;
            }
        }
    }
    if ($deregister_scripts) {
        foreach ($keylist as $key) {
            wp_dequeue_script($key);
        }
    }
}

function estatebud_url_rewrite()
{
    $property_page_id = get_option("estatebud_property_page_id", null);
    if($property_page_id){
        if ( function_exists('icl_object_id') ) {
            $property_page_id =  apply_filters( 'wpml_object_id', $property_page_id, 'post' );
        }
        $page = get_post($property_page_id);
        if($page instanceof WP_Post){
            $page_slug = $page->post_name;
            add_rewrite_tag('%property_id%', '([^&]+)');
            add_rewrite_tag('%is_property_details%', '([10]{1})');
            add_rewrite_rule("^$page_slug-([^/]+)/?", 'index.php?page_id='.$property_page_id.'&is_property_details=1&property_id=$matches[1]', "top");
            add_rewrite_rule("^$page_slug-([^/]+)/([^/]+)/?", 'index.php?page_id='.$property_page_id.'&is_property_details=1&property_id=$matches[1]', "top");
            flush_rewrite_rules();
        }
    }
}
add_action( "init", "estatebud_url_rewrite" );

function estatebud_inspect_scripts()
{
    global $wp_scripts;
    deregister_scripts($wp_scripts, "fancybox");
}
add_action('wp_print_scripts', 'estatebud_inspect_scripts', 100);

function estatebud_inspect_styles()
{
    global $wp_styles;
    deregister_styles($wp_styles, "fancybox");
}
add_action('wp_print_styles', 'estatebud_inspect_styles', 100);

function wpml_url_fix($languages)
{
    global $wp_query;
    $is_pretty_permalink = get_option("estatebud_pretty_permalinks", false);
    if (isset($wp_query->query["is_property_details"])) {
        $property_id = $wp_query->query["property_id"];
        foreach ($languages as $lang => $element) {
            if($is_pretty_permalink){
                // todo: will add translated slug after url
                $languages[$lang]["url"] = rtrim($element["url"], "/") . "-" . $property_id;
            }else{
                $languages[$lang]["url"] = rtrim($element["url"], "/") . "-" . $property_id;
            }
        }
    }
    return $languages;
}
add_filter('icl_ls_languages', 'wpml_url_fix');
